@@ -6,7 +6,7 @@
 
 from .classes import Expression, \
     Program, Node, Identifier, Factorize, Hyperedge, Variable, Sizing, State, \
-    Action, Auxiliary, MDPObjective, MDP, Time, Parameter
+    Action, Auxiliary, MDPObjective, MDP, Time, Parameter, Condition
 from .classes.error import RedefinitionError
 
 import numpy as np  # type: ignore
@@ -69,7 +69,6 @@ def semantic(program: Program) -> tuple:
     program.set_nb_var_index(global_index)
     program.set_variables_dict(program_variables_dict)
     program.set_global_parameters(global_param)
-
     return program, program_variables_dict, definitions
 
 
@@ -180,6 +179,7 @@ def check_and_extend_parameters_variables_in_nodes(nodes_list: list,
         _, nested_nodes_variables = get_layer_in_nested_dict(
             program_variables_dict, depth_list, only_dict=True)
         depth_list.remove(node_name)
+        add_node_in_variable_index(node)
         global_index = set_size_variables(node_variables,
                                           flat_branch_dictionary,
                                           global_index,
@@ -190,6 +190,15 @@ def check_and_extend_parameters_variables_in_nodes(nodes_list: list,
     return global_index
 
 
+def add_node_in_variable_index(node):
+    vars = node.get_variables()
+    for var in vars:
+        variable_identifier = var.get_identifier()
+        id_expression = variable_identifier.get_expression()
+        if id_expression is not None:
+            add_node_names_expression(id_expression, node, [])
+
+
 def check_and_extend_parameters_in_edges(edge_list: list,
                                          definitions: dict,
                                          depth_list: list):
@@ -266,7 +275,9 @@ def recursive_application_on_nodes_hyperedges(nodes,
         accumulator_dict.pop(node_name)
 
     for link in hyperedges:
-        function_hyperedge(link, variables, definitions)
+        accumulator_dict[link.get_name()] = definitions[link.get_name()]
+        function_hyperedge(link, variables, accumulator_dict)
+        del accumulator_dict[link.get_name()]
 
 
 def factorize_program(program, variables, definitions):
@@ -371,10 +382,6 @@ def check_expressions_dependency(node: Node,
 
         contains_var = predicate_variables_in_expression(expr, variables)
 
-        if contains_var is False:
-            error_(
-                'Objective only depends on constants not on variable at line '
-                + str(expr.get_line()))
         check_linear(expr, variables, parameters_obj)
 
 
@@ -757,9 +764,6 @@ def check_definition_expression(expr: Expression, variables_dict: dict,
                     seed_node_name != "global" and \
                     in_node_name != seed_node_name and \
                     seed_node_name not in parent_hood:
-                print(in_node_name)
-                print(parameters_dict)
-                print(variables_dict)
                 error_("ERROR: referencing variables "
                        "or parameters defined in node "
                        + str(seed_node_name) +
@@ -851,8 +855,8 @@ def add_node_names(node: Node):
 
         if condition is not None:
             condition_lhs, condition_rhs = condition.get_children()
-            add_node_names_expression(condition_lhs, node, reserved)
-            add_node_names_expression(condition_rhs, node, reserved)
+            add_node_names_condition(condition_lhs, node, reserved)
+            add_node_names_condition(condition_rhs, node, reserved)
 
         lhs = constraint.get_lhs()
         add_node_names_expression(lhs, node, reserved)
@@ -874,12 +878,20 @@ def add_node_names(node: Node):
         reserved = [index_var]
         if condition is not None:
             condition_lhs, condition_rhs = condition.get_children()
-            add_node_names_expression(condition_lhs, node, reserved)
-            add_node_names_expression(condition_rhs, node, reserved)
+            add_node_names_condition(condition_lhs, node, reserved)
+            add_node_names_condition(condition_rhs, node, reserved)
 
         add_node_names_expression(expr, node, reserved)
 
 
+def add_node_names_condition(condition, node, reserved):
+    if type(condition) == Expression:
+        add_node_names_expression(condition, node, reserved)
+    else:
+        for child in condition.get_children():
+            add_node_names_condition(child, node, reserved)
+
+
 def add_node_names_expression(expr: Expression, node,
                               reserved_identifiers=None):
     if reserved_identifiers is None:
@@ -901,6 +913,12 @@ def add_node_names_expression(expr: Expression, node,
             sum_children = sum_expr.get_children()
             time_interval = sum_expr.get_time_interval()
             index_name = time_interval.get_index_name()
+            begin = time_interval.get_begin()
+            step = time_interval.get_step()
+            end = time_interval.get_end()
+            add_node_names_expression(begin, node, reserved_identifiers)
+            add_node_names_expression(step, node, reserved_identifiers)
+            add_node_names_expression(end, node, reserved_identifiers)
             reserved_sum = reserved + [index_name]
             for child in sum_children:
                 add_node_names_expression(child, node, reserved_sum)
@@ -1853,13 +1871,16 @@ def parameter_evaluation(parameter_dict: dict, definitions: dict):
     input_definition = definitions.copy()
     for parameter_name in parameter_dict.keys():
         parameter = parameter_dict[parameter_name]
+        if parameter.read_from_file:
+            input_definition[parameter_name] = parameter
+            continue
         e = parameter.get_expression()
         if e is not None:
 
             value = e.evaluate_expression(input_definition)
-            value = [value]
+            if not isinstance(value, list):
+                value = [value]
         else:
-
             value = evaluate_table(parameter.get_vector(), input_definition)
 
         parameter.set_value(value)
@@ -1905,6 +1926,8 @@ def evaluate_table(list_values: list, definitions: dict) -> list:
                     error_('Parameter does not exit at this index : ' + str(
                         value_i))
                 value_i = values[index]
+        elif type(value) == Expression:
+            value_i = value.evaluate_expression(definitions)
         all_values.append(value_i)
     return all_values
 
