@@ -10,7 +10,7 @@ from .compiler.classes import Parameter, Expression, Node, Hyperedge, Time, \
     Program
 from .compiler.utils import error_, move_to_directory
 from .solver_api import cplex_solver, gurobi_solver, clp_solver, dsp_solver, \
-    xpress_solver
+    xpress_solver, highs_solver, cbc_solver
 from .output import generate_json
 
 from enum import Enum
@@ -66,10 +66,13 @@ class GbomlGraph:
         self.list_hyperedges = []
         self.timehorizon: Time = Time("T", Expression('literal', timehorizon))
         self.node_hyperedge_dict = {}
+        self.global_parameters = []
 
         self.program = None
-        self.matrix_a = None
-        self.matrix_b = None
+        self.matrix_a_ineq = None
+        self.matrix_b_ineq = None
+        self.matrix_a_eq = None
+        self.matrix_b_eq = None
         self.vector_c = None
         self.indep_term_c = None
 
@@ -202,9 +205,9 @@ class GbomlGraph:
            nb_processes (int) : number of processes used for model generation
 
         """
-
+        self.global_parameters = self.__remove_duplicates(self.global_parameters)
         program = Program(self.list_nodes, timescale=self.timehorizon,
-                          links=self.list_hyperedges)
+                          links=self.list_hyperedges, global_param=self.global_parameters)
         program, program_variables_dict, definitions = semantic(program)
         check_program_linearity(program, program_variables_dict, definitions)
         factorize_program(program, program_variables_dict, definitions)
@@ -214,13 +217,15 @@ class GbomlGraph:
         else:
             extend_factor(program, definitions)
 
-        matrix_a, vector_b = matrix_generation_a_b(program)
-        vector_c, indep_terms_c = matrix_generation_c(program)
+        matrix_eq, vector_b_eq, matrix_ineq, vector_b_ineq = matrix_generation_a_b(program)
+        vector_c, indep_terms_c, alone_term_c = matrix_generation_c(program)
         program.free_factors_objectives()
 
         self.program = program
-        self.matrix_a = matrix_a
-        self.matrix_b = vector_b
+        self.matrix_a_ineq = matrix_ineq
+        self.matrix_b_ineq = vector_b_ineq
+        self.matrix_a_eq = matrix_eq
+        self.matrix_b_eq = vector_b_eq
         self.vector_c = vector_c
         self.indep_term_c = indep_terms_c
 
@@ -247,7 +252,8 @@ class GbomlGraph:
         """
         vector_c = np.asarray(self.vector_c.sum(axis=0), dtype=float)
         objective_offset = float(self.indep_term_c.sum())
-        return solver_function(self.matrix_a, self.matrix_b, vector_c,
+        return solver_function(self.matrix_a_eq, self.matrix_b_eq,
+                               self.matrix_a_ineq, self.matrix_b_ineq, vector_c,
                                objective_offset, self.program.get_tuple_name())
 
     def solve_gurobi(self, opt_file: str = None, details=False):
@@ -268,18 +274,19 @@ class GbomlGraph:
 
         """
         gurobi_solver_function = \
-            lambda matrix_a, vector_b, vector_c, objective_offset, name_tuples:\
-                gurobi_solver(matrix_a, vector_b, vector_c, objective_offset,
-                              name_tuples, opt_file, details)
+            lambda matrix_a_eq, vector_b_eq, matrix_a_ineq, vector_b_ineq, vector_c, objective_offset, name_tuples: \
+            gurobi_solver(matrix_a_eq, vector_b_eq, matrix_a_ineq, vector_b_ineq, vector_c, objective_offset,
+                          name_tuples, opt_file, details)
         return self.__solve(gurobi_solver_function)
 
-    def solve_cplex(self, opt_file: str = None, details=False):
+    def solve_cplex(self, opt_file: str = None, details=False, opt_dict = None):
         """
         bound method solving the flattened optimization model with CPLEX
 
         Args:
             opt_file (str) : path to an optimization parameters file
             details (bool) : get variables and constraints information
+            opt_dict (dict): dictionary containing the optimization parameters
 
         Returns:
             solution (ndarray) : flattened solution
@@ -291,9 +298,9 @@ class GbomlGraph:
 
         """
         cplex_solver_function = \
-            lambda matrix_a, vector_b, vector_c, objective_offset, name_tuples:\
-                cplex_solver(matrix_a, vector_b, vector_c, objective_offset,
-                              name_tuples, opt_file, details)
+            lambda matrix_a_eq, vector_b_eq, matrix_a_ineq, vector_b_ineq, vector_c, objective_offset, name_tuples: \
+                cplex_solver(matrix_a_eq, vector_b_eq, matrix_a_ineq, vector_b_ineq, vector_c, objective_offset,
+                              name_tuples, opt_file, details, option_dict=opt_dict)
         return self.__solve(cplex_solver_function)
 
     def solve_xpress(self, opt_file: str = None, details=False):
@@ -313,14 +320,14 @@ class GbomlGraph:
             variables_information (dict) : dict of additional information concerning variables
         """
         xpress_solver_function = \
-            lambda matrix_a, vector_b, vector_c, objective_offset, name_tuples:\
-                xpress_solver(matrix_a, vector_b, vector_c, objective_offset,
+            lambda matrix_a_eq, vector_b_eq, matrix_a_ineq, vector_b_ineq, vector_c, objective_offset, name_tuples: \
+                xpress_solver(matrix_a_eq, vector_b_eq, matrix_a_ineq, vector_b_ineq, vector_c, objective_offset,
                               name_tuples, opt_file, details)
         return self.__solve(xpress_solver_function)
 
     def solve_clp(self):
         """
-        bound method solving the flattened optimization problem with Clp/Cbc
+        bound method solving the flattened optimization problem with Clp
 
         Returns:
             solution (ndarray) : flattened solution
@@ -331,6 +338,47 @@ class GbomlGraph:
         """
         return self.__solve(clp_solver)
 
+    def solve_cbc(self, opt_file= None, opt_dict= None):
+        """
+        bound method solving the flattened optimization problem with Cbc
+
+        Args:
+            opt_file (str): filename of file containing the optimization parameters
+            opt_dict (dict): dictionary containing the optimization parameters
+                             the key must be the parameter to tune
+                             the value a tuple of the <type, value>
+                             example: {"gap": ["double", 0.5]
+
+        Returns:
+            solution (ndarray) : flattened solution
+            objective (flat) : float of the objective value
+            status (str) : solver exit status
+            solver_info (dict) : dictionary storing solver information
+
+        """
+        if opt_dict is None:
+            opt_dict = dict()
+        cbc_solver_function = \
+            lambda matrix_a_eq, vector_b_eq, matrix_a_ineq, vector_b_ineq, vector_c, objective_offset, name_tuples: \
+                cbc_solver(matrix_a_eq, vector_b_eq, matrix_a_ineq, vector_b_ineq, vector_c, objective_offset,
+                           name_tuples, opt_file=opt_file, option_dict=opt_dict)
+
+        return self.__solve(cbc_solver_function)
+
+    def solve_highs(self):
+        """
+        bound method solving the flattened optimization problem with Highs
+
+        Returns:
+            solution (ndarray) : flattened solution
+            objective (flat) : float of the objective value
+            status (str) : solver exit status
+            solver_info (dict) : dictionary storing solver information
+
+        """
+
+        return self.__solve(highs_solver)
+
     def solve_dsp(self, algorithm="dw"):
         """
         bound method solving the optimization model with DSP
@@ -348,8 +396,8 @@ class GbomlGraph:
         """
 
         dsp_solver_function = \
-            lambda matrix_a, vector_b, vector_c, objective_offset, name_tuples:\
-                dsp_solver(matrix_a, vector_b, vector_c, objective_offset,
+            lambda matrix_a_eq, vector_b_eq, matrix_a_ineq, vector_b_ineq, vector_c, objective_offset, name_tuples:\
+                dsp_solver(matrix_a_eq, vector_b_eq, matrix_a_ineq, vector_b_ineq, vector_c, objective_offset,
                            name_tuples, self.program.
                            get_first_level_constraints_decomposition(),
                            algorithm=algorithm)
@@ -386,6 +434,73 @@ class GbomlGraph:
                              solution, objective, self.vector_c,
                              self.indep_term_c, constraint_info, variables_info)
 
+    def add_global_parameters_objects(self, global_parameters):
+        """
+        Add global parameters objects from list of parameters objects
+        Warning this function will be removed in future release !
+        Use add_global_parameters instead
+
+        Args:
+           global_parameters (list) : list of global parameters
+
+        Returns:
+
+        """
+        print("THE FUNCTION 'add_global_parameters_objects' WILL BE REMOVED SOON, use add_global_parameters instead")
+        for param in global_parameters:
+            if isinstance(param, Parameter):
+                self.global_parameters.append(param)
+
+    def add_global_parameters(self, global_parameters):
+        """
+        Add global parameters objects to the graph
+
+        Args:
+           global_parameters (list) : list of Parameter object or
+                                      tuples of <parameter_name, values> where values
+                                      can be an int/float/list <int/float> or a string to import from.
+
+        Returns:
+
+        """
+        for param in global_parameters:
+            if isinstance(param, Parameter):
+                self.global_parameters.append(param)
+            else:
+                identifier, value = param
+                self.add_global_parameter(identifier, value)
+
+    def add_global_parameter(self, identifier, value):
+        """
+        Add one global parameter objects to the graph
+
+        Args:
+           identifier (str) : parameter name
+           value (int|float|str) : value associated to the parameter (if string it expects a filename to read from)
+        Returns:
+
+        """
+        parameter = None
+        if isinstance(value, Parameter):
+            parameter = value
+        elif isinstance(value, str):
+            parameter = Parameter(identifier, value)
+        elif isinstance(value, float) or isinstance(value, int):
+            expr = Expression('literal', value)
+            parameter = Parameter(identifier, expr)
+        elif isinstance(value, list):
+            expression_values = []
+            expr = None
+            for val in value:
+                expr = Expression('literal', val)
+                expression_values.append(expr)
+            parameter = Parameter(identifier, None)
+            parameter.set_vector(expression_values)
+        else:
+            error_("Unaccepted type value for global parameter "
+                   + str(type(value)))
+        self.global_parameters.append(parameter)
+
     @staticmethod
     def import_all_nodes_and_edges(filename):
         """
@@ -397,6 +512,7 @@ class GbomlGraph:
         Returns:
             all_nodes (list) : list of nodes contained in file
             all_hyperedges (list) : list of hyperedges contained in file
+            all_global_param(list) : list of global parameters in file
 
         """
         old_dir, cut_filename = move_to_directory(filename)
@@ -406,12 +522,13 @@ class GbomlGraph:
         for node in all_nodes:
             to_return_nodes.append(deepcopy(node))
 
+        to_return_global_param = filename_graph.get_global_parameters()
         all_hyperedges = filename_graph.get_links()
         to_return_edges = []
         for edge in all_hyperedges:
             to_return_edges.append(deepcopy(edge))
         os.chdir(old_dir)
-        return to_return_nodes, to_return_edges
+        return to_return_nodes, to_return_edges, to_return_global_param
 
     @staticmethod
     def import_node(filename: str, *imported_node_identifier: str,
@@ -504,6 +621,22 @@ class GbomlGraph:
         os.chdir(old_dir)
         return imported_hyperedge
 
+    @staticmethod
+    def create_parameter(parameter_name, value):
+        """
+        static method that returns a parameter whose name is given by parameter_name
+        and expression by value
+
+        Args:
+           parameter_name (str) : parameter name
+           value (float/int) : value of parameter
+
+        Returns:
+            param (Parameter): parameter created
+
+        """
+        return Parameter(parameter_name, Expression("literal", value))
+
     @staticmethod
     def rename(node_or_hyperedge, new_name):
         """
@@ -582,7 +715,7 @@ class GbomlGraph:
            in_node (Node) : node to which sub-hyperedge should be added
 
         """
-        in_node.add_link(hyperedge_to_add)
+        in_node.add_sub_hyperedge(hyperedge_to_add)
         in_node.update_internal_dict()
 
     @staticmethod
@@ -698,7 +831,7 @@ class GbomlGraph:
                                                            parameter_name,
                                                            value)
             elif isinstance(value, list):
-                GbomlGraph.redefine_parameter_from_values(node_or_hyperedge,
+                GbomlGraph.__redefine_parameter_from_values(node_or_hyperedge,
                                                           parameter_name,
                                                           value)
             else:
@@ -780,3 +913,18 @@ class GbomlGraph:
         """
         change_tuple = [old_node_name, new_node_name, None]
         hyperedge.add_name_change(change_tuple)
+
+    @staticmethod
+    def __remove_duplicates(list_globals):
+        seen_param = dict()
+        output_list = []
+        for index, param_global in enumerate(list_globals):
+            name = param_global.get_name()
+            if name in seen_param:
+                index = seen_param[name]
+                output_list.pop(index)
+                output_list.insert(index, param_global)
+            else:
+                output_list.append(param_global)
+                seen_param[name] = index
+        return output_list
