@@ -22,6 +22,7 @@ into respectively a json and csv file.
 """
 
 import csv
+import numpy as np
 from gboml.version import __version__
 
 
@@ -71,8 +72,8 @@ def get_nodes_and_edges_model_information(nodes, edges):
         node_data["number_parameters"] = node.get_number_parameters()
         node_data["number_variables"] = node.get_number_variables()
         node_data["number_constraints"] = node.get_number_constraints()
-        node_data["number_expanded_constraints"] = \
-            node.get_number_expanded_constraints()
+        nb_eq, nb_ineq = node.get_number_expanded_constraints()
+        node_data["number_expanded_constraints"] = nb_eq+nb_ineq
         node_data["number_objectives"] = node.get_number_objectives()
         node_data["number_expanded_objectives"] = \
             node.get_number_expanded_objectives()
@@ -94,8 +95,9 @@ def get_nodes_and_edges_model_information(nodes, edges):
         link_data = dict()
         link_data["number_parameters"] = link.get_number_parameters()
         link_data["number_constraints"] = link.get_number_constraints()
-        link_data["number_expanded_constraints"] = \
-            link.get_number_expanded_constraints()
+        nb_eq, nb_ineq = link.get_number_expanded_constraints()
+        link_data["number_expanded_constraints"] = nb_ineq+nb_eq
+
         link_data["parameters"] = \
             convert_parameter_dict_to_values(link.get_parameter_dict())
         link_data["variables_used"] = link.get_variables_used()
@@ -174,14 +176,16 @@ def dict_values_in_nested_dict(nodes, hyperedges, solution,
         constraints_info_in_node = node.get_constraints_data()
 
         if constraint_info != {}:
-            for constraint_name, indexes in constraints_info_in_node.items():
-                all_info_for_constraint = {}
-                for constraint_info_name, constraint_info_values in \
-                        constraint_info.items():
-                    all_info_for_constraint[constraint_info_name] = \
-                        constraint_info_values[indexes]
-                dict_constraints_solution[constraint_name] = \
-                    all_info_for_constraint
+            for constr_by_type, in_node_constr_by_type in constraints_info_in_node.items():
+                current_dict = constraint_info[constr_by_type]
+                for constraint_name, indexes in in_node_constr_by_type.items():
+                    all_info_for_constraint = {}
+                    for constraint_info_name, constraint_info_values in \
+                            current_dict.items():
+                        all_info_for_constraint[constraint_info_name] = \
+                            constraint_info_values[indexes]
+                    dict_constraints_solution[constraint_name] = \
+                        all_info_for_constraint
 
             if dict_constraints_solution != {}:
                 dict_node_solution['constraints'] = dict_constraints_solution
@@ -192,10 +196,14 @@ def dict_values_in_nested_dict(nodes, hyperedges, solution,
         objectives_data = node.get_objectives_data()
         for i, objective_info in objectives_data.items():
             objective_type = objective_info["type"]
-            indexes = objective_info["indexes"]
-            objective_value = c_matrix_time_solution[indexes].sum()
-            if objective_type == "max":
-                objective_value = -objective_value
+
+            if objective_type == "min" or objective_type == "max":
+                indexes = objective_info["indexes"]
+                objective_value = c_matrix_time_solution[indexes].sum()
+                if objective_type == "max":
+                    objective_value = -objective_value
+            else:
+                objective_value = objective_info["values"]
 
             if "name" in objective_info:
                 name_objective = objective_info["name"]
@@ -233,14 +241,17 @@ def dict_values_in_nested_dict(nodes, hyperedges, solution,
             dict_hyperedge_solution = {}
             dict_hyperedge_constraints_solution = {}
             constraints_info_in_node = hyperedge.get_constraints_data()
-            for constraint_name, indexes in constraints_info_in_node.items():
-                all_info_for_constraint = {}
-                for constraint_info_name, constraint_info_values in \
-                        constraint_info.items():
-                    all_info_for_constraint[constraint_info_name] = \
-                        constraint_info_values[indexes]
-                dict_hyperedge_constraints_solution[constraint_name] = \
-                    all_info_for_constraint
+
+            for constr_type, in_node_dict_by_type in constraints_info_in_node.items():
+                current_dict = constraint_info[constr_type]
+                for constraint_name, indexes in in_node_dict_by_type.items():
+                    all_info_for_constraint = {}
+                    for constraint_info_name, constraint_info_values in \
+                            current_dict.items():
+                        all_info_for_constraint[constraint_info_name] = \
+                            constraint_info_values[indexes]
+                    dict_hyperedge_constraints_solution[constraint_name] = \
+                        all_info_for_constraint
 
             if dict_hyperedge_constraints_solution != {}:
                 dict_hyperedge_solution['constraints'] = \
@@ -310,7 +321,7 @@ def generate_json(program, solver_data, status, solution, objective, c_matrix,
             dict_values_in_nested_dict(program.get_nodes(),
                                        program.get_links(),
                                        solution,
-                                        product,
+                                       product,
                                        constraint_info,
                                        variables_info)
     gathered_data["solution"] = solution_data
@@ -318,7 +329,7 @@ def generate_json(program, solver_data, status, solution, objective, c_matrix,
     return gathered_data
 
 
-def flat_graph_and_add_node_prefix(nodes, hyperedges, solution, prefix=""):
+def flat_graph_and_add_node_prefix(nodes, hyperedges, solution, product, constraints_info=None, prefix=""):
     """flat_graph_and_add_node_prefix
 
         creates a list of the nodes and hyperedges variables & parameters
@@ -329,12 +340,15 @@ def flat_graph_and_add_node_prefix(nodes, hyperedges, solution, prefix=""):
             nodes (list<Node>): list of nodes
             hyperedges (list<Hyperedge>): list of hyperedges
             solution (array): flat array containing the problem's solution
+            product (ndarray): product of the solution and objective matrix
+            constraints_info (dict): dict of constraints information
             prefix (str): prefix to add to the variables and parameters names
 
         Returns:
             name_tuples: list of tuples <identifier, values>
 
     """
+    solution = np.array(solution)
     name_tuples = []
     for node in nodes:
         variables_dict = node.get_dictionary_variables()
@@ -348,34 +362,75 @@ def flat_graph_and_add_node_prefix(nodes, hyperedges, solution, prefix=""):
                               (start_index + size_variable)].flatten().tolist()
             name_tuples.append([identifier_name, values])
 
-        parameters = node.get_parameter_dict()
-        for parameter_name, parameter_object in parameters.items():
+        #parameters = node.get_parameter_dict()
+        #for parameter_name, parameter_object in parameters.items():
 
-            values = parameter_object.get_value()
-            parameter_full_name = str(node_name) + "." + str(parameter_name)
-            name_tuples.append([parameter_full_name, values])
+        #    values = parameter_object.get_value()
+        #    parameter_full_name = str(node_name) + "." + str(parameter_name)
+        #    name_tuples.append([parameter_full_name, values])
+
+        objectives_data = node.get_objectives_data()
+        for i, objective_info in objectives_data.items():
+            objective_type = objective_info["type"]
+            if "name" in objective_info:
+                name_objective = objective_info["name"]
+
+            else:
+                name_objective = "unnamed_objective"
+
+            if objective_type == "min" or objective_type == "max":
+                indexes = objective_info["indexes"]
+                objective_value = product[indexes].sum()
+                if objective_type == "max":
+                    objective_value = -objective_value
+            # ELSE : objective_type is no_variables
+            else:
+                objective_value = objective_info["values"]
+            name_tuples.append([str(node_name) + "." + name_objective, [objective_value]])
+
+        constraints_info_in_node = node.get_constraints_data()
+        if constraints_info != {}:
+            for constr_type, constr_dict in constraints_info_in_node.items():
+                current_constr_dict = constraints_info[constr_type]
+                for constraint_name, indexes in constr_dict.items():
+                    for constraint_info_name, constraint_info_values in \
+                            current_constr_dict.items():
+                        name_tuples.append([str(node_name) + "." + constraint_name + "." + constraint_info_name,
+                                            constraint_info_values[indexes]])
 
         sub_nodes = node.get_sub_nodes()
         sub_hyperedges = node.get_sub_hyperedges()
         name_tuples += flat_graph_and_add_node_prefix(sub_nodes,
                                                       sub_hyperedges,
                                                       solution,
+                                                      product,
+                                                      constraints_info,
                                                       node_name+".")
 
     for hyperedge in hyperedges:
         hyperedge_name = prefix+hyperedge.get_name()
         parameters = hyperedge.get_parameter_dict()
-        for parameter_name, parameter_object in parameters.items():
-            values = parameter_object.get_value()
-            parameter_full_name = str(hyperedge_name) + "." \
-                                  + str(parameter_name)
-            name_tuples.append([parameter_full_name, values])
+        #for parameter_name, parameter_object in parameters.items():
+        #    values = parameter_object.get_value()
+        #    parameter_full_name = str(hyperedge_name) + "." \
+        #                          + str(parameter_name)
+        #    name_tuples.append([parameter_full_name, values])
+
+        if constraints_info != {}:
+            constraints_info_in_hyperedge = hyperedge.get_constraints_data()
+            for constr_type, constr_dict in constraints_info_in_hyperedge.items():
+                current_constr_dict = constraints_info[constr_type]
+                for constraint_name, indexes in constr_dict.items():
+                    for constraint_info_name, constraint_info_values in \
+                            current_constr_dict.items():
+                        name_tuples.append([str(hyperedge_name) + "." + constraint_name + "." + constraint_info_name,
+                                            constraint_info_values[indexes]])
 
     return name_tuples
 
 
-def generate_list_values_tuple(program, solution):
-    """generate_pandas
+def generate_list_values_tuple(program, solution, c_matrix, indep_terms_c, constraints_info=None):
+    """generate_list_values_tuple
 
         Converts all the information contained in the inputs into a tuple
         of names and values where values[i] is the list of values
@@ -385,27 +440,37 @@ def generate_list_values_tuple(program, solution):
             program (Program): program object containing the augmented
                                abstract syntax tree
             solution (array): flat array containing the problem's solution
-
+            c_matrix(coo-matrix): matrix of all objectives
+            indep_terms_c (array): array of independant terms
+            constraints_info(dict): dict of information related to constraints
         Returns:
             names: list of flatten names of all parameters and variables
             values : list of values
 
     """
+    if constraints_info == None:
+        constraints_info = dict()
     ordered_values = []
     names = []
     nodes = program.get_nodes()
     hyperedges = program.get_links()
 
+    product = []
+    if solution is not None:
+        product = c_matrix * solution + indep_terms_c
+
     global_param: dict = program.get_global_parameters()
-    for param in global_param.keys():
-        values = global_param[param].get_value()
-        full_name = "global." + str(param)
-        names.append(full_name)
-        ordered_values.append(values)
+    #for param in global_param.keys():
+    #    values = global_param[param].get_value()
+    #    full_name = "global." + str(param)
+    #    names.append(full_name)
+    #    ordered_values.append(values)
 
     name_value_tuples = flat_graph_and_add_node_prefix(nodes,
                                                        hyperedges,
                                                        solution,
+                                                       product,
+                                                       constraints_info,
                                                        prefix="")
 
     for name, values in name_value_tuples:
